package com.blueparabola.services
{
	import com.adobe.serializers.json.JSONDecoder;
	import com.adobe.serializers.json.JSONEncoder;
	import com.adobe.serializers.json.JSONSerializationFilter;
	
	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import mx.collections.ArrayCollection;
	import mx.formatters.Formatter;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;
	import mx.utils.ObjectProxy;
	
	import spark.formatters.CurrencyFormatter;

	[Event(name="loaded", type="flash.events.Event")]
	[Event(name="fault", type="flash.events.Event")]
	
	[Event(name="authFail", type="flash.events.Event")]
	[Event(name="authSucceed", type="flash.events.Event")]
	
	public class SalesService extends EventDispatcher
	{
		public static const LOADED:String = "loaded";
		public static const FAULT:String = "fault";
		
		public static const AUTH_SUCCEED:String = "authSucceed";
		public static const AUTH_FAIL:String = "authFail";
		
		// Input
		
		[Bindable]
		public var startDate:Date;
		
		[Bindable]
		public var endDate:Date;
		
		[Bindable]
		public var countries:Array;
		
		[Bindable]
		public var handlers:Array;
		
		[Bindable]
		public var SKUs:Array;
		
		// Output
		
		protected var _resultsByCountry:ArrayCollection;
		protected var _resultsByCategory:ArrayCollection;
		protected var _resultsBySKU:ArrayCollection;
		
		[Bindable(event="loaded")]
		public function get resultsByCountry():ArrayCollection {
			return _resultsByCountry;
		}
		
		[Bindable(event="loaded")]
		public function get resultsByCategory():ArrayCollection {
			return _resultsByCategory;
		}
		
		[Bindable(event="loaded")]
		public function get resultsBySKU():ArrayCollection {
			return _resultsBySKU;
		}
		
		[Bindable]
		public var loading:Boolean = false;
		
		// Internal variables
		
		protected var _httpService:HTTPService;
		
		protected static var _sessionId:String;
		
		// Constructor
		
		public function SalesService() {
			super();
			
			_httpService = new HTTPService();
			_httpService.url = "https://localhost/flex-json.php";
			_httpService.method = "POST";
			_httpService.addEventListener(ResultEvent.RESULT, handleReceivedData);
			_httpService.addEventListener(FaultEvent.FAULT, handleFaultEvent);
		}
		
		// Public method to handle administrator login
		
		public function login(username:String, password:String):void {
			loading = true;
			
			var encoder:JSONEncoder = new JSONEncoder;
			var localHttpService:HTTPService = new HTTPService();
			
			localHttpService.url = "https://localhost/flex-json.php";
			localHttpService.method = "POST";
			
			var parameters:Array = [
				username,
				password
			];
			
			var payload:Object = {
				c : "User::UserServices",
				m : "adminLogin",
				p : parameters,
				s : SalesService._sessionId,
					t : "AoGBAIkPhQnBckyo9mbv2s5ObN3f/bBgjUwL2l7hvWm11Anrq9fut9TzUYGasgXu"
			};
			
			localHttpService.addEventListener(ResultEvent.RESULT, function(e:ResultEvent):void {
				var decoder:JSONDecoder = new JSONDecoder;
				
				var result:Object = decoder.decode(localHttpService.lastResult.toString());
				
				if (result.f) {
					trace(result.r.m);
					dispatchEvent(new Event(SalesService.AUTH_FAIL));
				} else {
					SalesService._sessionId = result.s;
					dispatchEvent(new Event(SalesService.AUTH_SUCCEED));
				}
				
				loading = false;
			});
			
			localHttpService.addEventListener(FaultEvent.FAULT, function(e:FaultEvent):void {
				dispatchEvent(new Event(SalesService.AUTH_FAIL));
				loading = false;
			});
			
			localHttpService.send({"data" : encoder.encode(payload)});
		}
		
		// Public method to start loading
		
		public function load():void {
			loading = true;
			
			if (_httpService) {
				_httpService.cancel();
			}
			
			var parameters:Array = [
				startDate.toString(),
				endDate.toString(),
				countries,
				handlers,
				SKUs
			];
			
			var payload:Object = {
				c : "Store::StatService",
				m : "getSalesBreakdown",
				p : parameters,
				s : SalesService._sessionId,
				t : "AoGBAIkPhQnBckyo9mbv2s5ObN3f/bBgjUwL2l7hvWm11Anrq9fut9TzUYGasgXu"
			};
			
			var encoder:JSONEncoder = new JSONEncoder;
			
			_httpService.send({"data" : encoder.encode(payload)});
		}
		
		// Result handler
		
		protected function handleReceivedData(e:ResultEvent):void {
			var decoder:JSONDecoder = new JSONDecoder;
			
			var result:Object = decoder.decode(_httpService.lastResult.toString());
			
			if (result.f) {
				trace(result.r.m);
				dispatchEvent(new Event(SalesService.FAULT));
			} else {
				parseResult(result.r);
			}
			
			loading = false;
		}
		
		// Result parser
		
		protected function parseResult(resultCollection:ArrayCollection):void {
			var countryResults:Object = {};
			var categoryResults:Object = {};
			var skuResults:Object = {};
			
			for each (var result:ObjectProxy in resultCollection) {
				
				// SKU
				
				var bin:Object = skuResults[result.sku];
				
				if (!bin) {
					bin = {
						name		: result.productName,
						itemCount 	: 0,
						total		: 0
					};
					
					skuResults[result.sku] = bin;
				}
				
				bin.itemCount += Number(result.quantity);
				bin.total += Number(result.price) * Number(result.quantity);
				
				// Country
				
				if (!result.country) {
					result.country = "Unknown";
				}
				
				bin = countryResults[result.country];
				
				if (!bin) {
					bin = {
						name		: result.country,
						itemCount	: 0,
						total		: 0
					};
					
					countryResults[result.country] = bin;
				}
				
				bin.itemCount += Number(result.quantity);
				bin.total += Number(result.price) * Number(result.quantity);
				
				// Category
				
				bin = categoryResults[result.handlerName];
				
				if (!bin) {
					bin = {
						name		: result.handlerName,
						itemCount	: 0,
						total		: 0
					};
					
					categoryResults[result.handlerName] = bin;
				}
				
				bin.itemCount += Number(result.quantity);
				bin.total += Number(result.price) * Number(result.quantity);
			}
			
			// Generate the final output
			
			var countryArray:Array = new Array();
			var categoryArray:Array = new Array();
			var skuArray:Array = new Array();
			
			var formatter:CurrencyFormatter = new CurrencyFormatter();
			
			formatter.useCurrencySymbol = true;
			
			for each (var entry:Object in countryResults) {
				entry.description = "Sales: " + entry.itemCount + " — Revenue: " + formatter.format(entry.total); 
				countryArray.push(entry);
			}

			for each (entry in skuResults) {
				entry.description = "Sales: " + entry.itemCount + " — Revenue: " + formatter.format(entry.total); 
				skuArray.push(entry);
			}

			for each (entry in categoryResults) {
				entry.description = "Sales: " + entry.itemCount + " — Revenue: " + formatter.format(entry.total); 
				categoryArray.push(entry);
			}
			
			// Populate output
			
			var x:Object =skuArray.sortOn("total", Array.DESCENDING | Array.NUMERIC); 

			_resultsByCountry = new ArrayCollection(countryArray.sortOn("total", Array.DESCENDING | Array.NUMERIC));
			_resultsByCategory = new ArrayCollection(categoryArray.sortOn("total", Array.DESCENDING | Array.NUMERIC));
			_resultsBySKU = new ArrayCollection(skuArray.sortOn("total", Array.DESCENDING | Array.NUMERIC));
			
			// Notify bindings of result change
			
			dispatchEvent(new Event(LOADED));
		}
		
		// Error handler
		
		protected function handleFaultEvent(e:FaultEvent):void {
			trace(e);
			
			loading = false;
		}
	}
}